import { Component ,ViewChild} from '@angular/core';
import * as wjCore from 'wijmo/wijmo';
import * as wjInput from 'wijmo/wijmo.input';
import * as wjGrid from 'wijmo/wijmo.grid';
import * as wjFlexSheet from 'wijmo/wijmo.grid.sheet';
import { Hero } from './hero';
import { HeroService } from './hero.service';
@Component({
  
  selector: 'app-root',
  template: `
<h3>{{title}}</h3>
Right click on FlexSheet rowHeader for deleting row
    <div>
        <div style="width:500px;float:left;">
            <wj-flex-sheet #flex style="width:500px;height:500px;" [allowAddNew]="true">
                 <wj-sheet [name]="'Heroes'" [itemsSource]="heroes"></wj-sheet>
            </wj-flex-sheet>
        </div>
        <div style="width:400px;padding:100px;float:left;">
            <button (click)="updateHero()">Get updatedHeroList</button>
            <p *ngFor="let hero of updatedHeroes"> <b>{{hero.id}}</b> <i>{{hero.name}}</i>  </p>
        <div>
    </div>

  `,
  styleUrls: ['./app.component.css']
})
export class AppComponent {
    title = 'FlexSheet CRUD operations';
    heroes: wjCore.CollectionView;
    public updatedHeroes : Hero[]
    @ViewChild('flex') flex: wjFlexSheet.FlexSheet;
    constructor(private heroService: HeroService) {
        this.updateHero();
        this.heroService.getHeroes()
            .then(heroes => {
                this.heroes = new wjCore.CollectionView(heroes);
                console.log(this.heroes);
                this.heroes.trackChanges = true;
                this.heroes.collectionChanged.addHandler((s, e) => {

                    if (this.heroes.itemsEdited.length > 0 && this.heroes.itemsEdited[0].id !== undefined) {
                        console.log('updated edit item');
                        this.heroService.update(this.heroes.itemsEdited[0]).then(() => { console.log('updated'); });
                        this.heroes.itemsEdited.clear();
                    }
                    if (this.heroes.itemsAdded.length > 0 && this.heroes.itemsAdded[0].name !== '') {
                        this.heroService.create(this.heroes.itemsAdded[0].name).then(hero => {
                            // update collection view
                            this.heroService.getHeroes().then(heroes => this.heroes.sourceCollection = heroes);
                            //clear addedItems array
                            this.heroes.itemsAdded.clear();
                            console.log('Added new record', this.heroes, hero);
                        });
                    }
                    if (this.heroes.itemsRemoved.length > 0) {
                        this.heroService.delete(this.heroes.itemsRemoved[0].id).then(() => {
                            //clear removedItem array
                            this.heroes.itemsRemoved.clear();
                            console.log('Deleted deleted record ');
                        });
                    }
                });
            });

    }
    createContextMenu(options, callback) {
        var menu = document.createElement('div'),
            lb = new wjInput.ListBox(menu);
        lb.itemsSource = options;
        lb.selectedIndex = -1;
        menu.addEventListener('click', function (e) {
            menu.style.visibility = 'hidden';
            callback(lb.selectedItem);
            lb.selectedIndex = -1;
        });
        menu.addEventListener('keydown', function (e) {
            if (e.keyCode == wjCore.Key.Escape) {
                menu.style.visibility = 'hidden';
            }
        });
        menu.addEventListener('blur', function (e) {
            setTimeout(function () {
                menu.style.visibility = 'hidden';
            }, 300);
        });
        return menu;
    }
    showPopup(popup, ref) {

        // make sure popup is a child of the body
        if (!popup.parentElement) {
            document.body.appendChild(popup);
        }

        // calculate popup position
        var sz = popup.getBoundingClientRect(),
            left = ref.clientX - sz.width / 2,
            top = ref.clientY - sz.height / 2;

        // make sure popup is visible
        left = Math.max(0, Math.min(left, innerWidth - sz.width));
        top = Math.max(0, Math.min(top, innerHeight - sz.height));

        // show the popup
        wjCore.setCss(popup, {
            position: 'absolute',
            left: left + pageXOffset,
            top: top + pageYOffset,
            visibility: 'visible'
        });

        // focus on the popup
        popup.focus();
    }
    ngAfterViewInit(): void {
        if (this.flex) {
            var menu = this.createContextMenu('Delete Row'.split(','), (option) => {
                this.heroes.remove(this.flex.selectedItems[0]);
            });

            this.flex.rowHeaders.hostElement.addEventListener('contextmenu', (e) => {
                this.showPopup(menu, e);
            });
            this.flex.rowEditEnding.addHandler((s, e: wjGrid.CellRangeEventArgs) => {
                if (s.rows[s.rows.length - 2].dataItem.name == "") {
                    this.heroes.cancelNew();
                }
            });
        }
    }

    updateHero() {
        this.heroService.getHeroes().then(heroes => { this.updatedHeroes = heroes });
    }
}


/*
Copyright 2017 Google Inc. All Rights Reserved.
Use of this source code is governed by an MIT-style license that
can be found in the LICENSE file at http://angular.io/license
*/